==> modelsummary_0.4.1.tar.gz: modelsummary/R/extract.R <==
#' Extract and combine estimates and goodness-of-fit statistics from several
#' statistical models.
#'
#' @inheritParams modelsummary
#' @return tibble
#' @examples
#' library(modelsummary)
#' data(trees)
#' models <- list()
#' models[['Bivariate']] <- lm(Girth ~ Height, data = trees)
#' models[['Multivariate']] <- lm(Girth ~ Height + Volume, data = trees)
#' extract(models)
#'
#' @export
extract <- function(models,
                    statistic = 'std.error',
                    statistic_override = NULL,
                    statistic_vertical = TRUE,
                    conf_level = 0.95,
                    coef_map = NULL,
                    coef_omit = NULL,
                    gof_map = modelsummary::gof_map,
                    gof_omit = NULL,
                    add_rows = NULL,
                    stars = FALSE,
                    fmt = '%.3f',
                    estimate = 'estimate',
                    add_rows_location = NULL,
                    ...) {

    # models must be a list of models
    if (!'list' %in% class(models)) {
        models <- list(models)
    }

    # sanity check functions are hosted in R/sanity_checks.R
    sanity_statistic(statistic, statistic_override, statistic_vertical, models)
    sanity_conf_level(conf_level)
    sanity_coef_map(coef_map)
    sanity_coef_omit(coef_omit)
    sanity_gof_map(gof_map)
    sanity_gof_omit(gof_omit)
    sanity_add_rows_location(add_rows_location) # deprecated 
    sanity_add_rows(add_rows, models)
    sanity_stars(stars)
    sanity_fmt(fmt)
    sanity_estimate(estimate)

    # model names
    if (is.null(names(models))) {
        model_names <- paste('Model', 1:length(models))
    } else {
        if (any(names(models) == '')) {
            stop('Model names cannot include empty strings. Please make sure that every object in the `models` list has a unique, non-empty name. If the `models` list has no names at all, `modelsummary` will create some automatically.')
        }
        model_names <- names(models)
    }

    # if statistics_override is a single function, repeat it in a list to allow map
    if (is.function(statistic_override)) {
        statistic_override <- rep(list(statistic_override), length(models))
    }

    # extract and combine estimates
    est <- list()
    for (i in seq_along(models)) {
        est[[i]] <- extract_estimates(model = models[[i]],
                                      fmt = fmt,
                                      estimate = estimate,
                                      statistic = statistic,
                                      statistic_override = statistic_override[[i]],
                                      statistic_vertical = statistic_vertical,
                                      conf_level = conf_level,
                                      stars = stars,
                                      ...)

        # coef_map: omit, rename (must be done before join to collapse rows)
        if (!is.null(coef_map)) {
            est[[i]] <- est[[i]][est[[i]]$term %in% names(coef_map),] # omit (white list)
            idx <- match(est[[i]]$term, names(coef_map))
            est[[i]]$term <- coef_map[idx] # rename

            # defensive programming
            if (any(table(est[[i]]$term) > 2)) {
                msg <- paste('You are trying to assign the same name to two different variables in model', i)
                stop(msg)
            }
        }
        # coef_omit
        if (!is.null(coef_omit)) {
            est[[i]] <- est[[i]] %>%
                        dplyr::filter(!stringr::str_detect(term, coef_omit))
        }

        # set model name
        colnames(est[[i]])[3] <- model_names[i]
    }

    est <- est %>% 
           purrr::reduce(dplyr::full_join, by = c('term', 'statistic'))  %>%
           dplyr::mutate(group = 'estimates') %>%
           dplyr::select(group, term, statistic, names(.))

    # reorder estimates (must be done after join
    if (!is.null(coef_map)) {
        idx <- match(est[['term']], coef_map)
        est <- est[order(idx, est[['statistic']]),]
    }

    # extract and combine gof
    gof <- models %>%
           purrr::map(extract_gof, fmt = fmt, gof_map = gof_map)  %>%
           purrr::reduce(dplyr::full_join, by = 'term') %>%
           stats::setNames(c('term', model_names))

    # add gof row identifier
    gof <- gof %>%
           dplyr::mutate(group = 'gof') %>%
           dplyr::select(group, term, names(.))

    # omit gof using regex
    if (!is.null(gof_omit)) {
        gof <- gof %>%
               dplyr::filter(!stringr::str_detect(term, gof_omit))
    }

    # gof_map: omit, reorder, rename
    gof <- gof[!gof$term %in% gof_map$raw[gof_map$omit],] # omit (black list)
	gof_names <- gof_map$clean[match(gof$term, gof_map$raw)] # rename
	gof_names[is.na(gof_names)] <- gof$term[is.na(gof_names)]
    gof$term <- gof_names
	idx <- match(gof$term, gof_map$clean) # reorder
    gof <- gof[order(idx, gof$term),] 

    # add_rows: this needs to be done after sorting and combining to preserve
    # user-selected row order. We need the 'section' information otherwise we
    # won't know where to draw the line which separates estimates from gof. 
    if (!is.null(add_rows)) {

        if (inherits(add_rows, 'list')) {
            add_rows <- do.call('rbind', add_rows) %>%
                        data.frame(stringsAsFactors = FALSE) %>%
                        stats::setNames(colnames(gof)[2:ncol(gof)]) %>%
                        dplyr::mutate(section = 'bottom', position = 1e6)
        }

        for (i in 1:nrow(add_rows)) {
            newrow <- add_rows[i, ] %>% dplyr::select(-section, -position)
            if (add_rows$section[i] == 'bottom') {
                newrow <- newrow %>% dplyr::mutate(group = 'gof') 
                gof <- gof %>%
                       tibble::add_row(newrow, .before = add_rows$position[i])
            } else {
                newrow <- newrow %>% 
                          dplyr::mutate(group = 'estimates',
                                 statistic = 'estimate') 
                est <- est %>%
                       tibble::add_row(newrow, .before = add_rows$position[i])
            }
        }

    }

    # combine estimates and gof
    if (nrow(gof) > 0) {
        tab <- dplyr::bind_rows(est, gof)
    } else {
        tab <- est
    }

    # empty cells
    tab[is.na(tab)] <- ''

    # output
    return(tab)
}

==> modelsummary_0.5.1.tar.gz: modelsummary/R/extract.R <==
#' Extract and combine estimates and goodness-of-fit statistics from several
#' statistical models.
#'
#' @inheritParams modelsummary
#' @return tibble
#' @export
extract <- function(models,
                    statistic = 'std.error',
                    statistic_override = NULL,
                    statistic_vertical = TRUE,
                    conf_level = 0.95,
                    coef_map = NULL,
                    coef_omit = NULL,
                    gof_map = modelsummary::gof_map,
                    gof_omit = NULL,
                    stars = FALSE,
                    fmt = '%.3f',
                    estimate = 'estimate',
                    ...) {

    # models must be a list of models
    if (!'list' %in% class(models)) {
        models <- list(models)
    }

    # sanity check functions are hosted in R/sanity_checks.R
    sanity_statistic(statistic, statistic_override, statistic_vertical, models)
    sanity_conf_level(conf_level)
    sanity_coef_map(coef_map)
    sanity_coef_omit(coef_omit)
    sanity_gof_map(gof_map)
    sanity_gof_omit(gof_omit)
    sanity_stars(stars)
    sanity_fmt(fmt)
    sanity_estimate(estimate)

    # model names
    if (is.null(names(models))) {
        model_names <- paste('Model', 1:length(models))
    } else {
        model_names <- names(models)
    }
    model_names <- pad(model_names)

    # if statistics_override is a single function, repeat it in a list to allow
    # map
    if (is.function(statistic_override)) {
        statistic_override <- rep(list(statistic_override), length(models))
    }

    # extract and combine estimates
    est <- list()
    for (i in seq_along(models)) {
        est[[i]] <- extract_estimates(model = models[[i]],
                                      fmt = fmt,
                                      estimate = estimate,
                                      statistic = statistic,
                                      statistic_override = statistic_override[[i]],
                                      statistic_vertical = statistic_vertical,
                                      conf_level = conf_level,
                                      stars = stars,
                                      ...)

        # coef_map: omit, rename (must be done before join to collapse rows)
        if (!is.null(coef_map)) {
            est[[i]] <- est[[i]][est[[i]]$term %in% names(coef_map),] # omit (white list)
            idx <- match(est[[i]]$term, names(coef_map))
            est[[i]]$term <- coef_map[idx] # rename

            # defensive programming
            # make sure no duplicate estimate names *within* a single model.
            # this cannot be in input sanity checks. idx paste allows multiple
            # statistics
            idx <- paste(est[[i]]$term, est[[i]]$statistic)
            if (anyDuplicated(idx) > 2) {
                stop('Two coefficients from a single model cannot share the same name. Check model ', i)
            }
        }

        # coef_omit
        if (!is.null(coef_omit)) {
            est[[i]] <- est[[i]] %>%
                        dplyr::filter(!stringr::str_detect(term, coef_omit))
        }

        # set model name
        colnames(est[[i]])[3] <- model_names[i]
    }

    est <- est %>% 
           purrr::reduce(dplyr::full_join, by = c('term', 'statistic'))  %>%
           dplyr::mutate(group = 'estimates') %>%
           dplyr::select(group, term, statistic, names(.))

    # reorder estimates (must be done after join
    if (!is.null(coef_map)) {
        idx <- match(est[['term']], coef_map)
        est <- est[order(idx, est[['statistic']]),]
    }

    # extract and combine gof
    gof <- models %>%
           purrr::map(extract_gof, fmt = fmt, gof_map = gof_map)  %>%
           purrr::reduce(dplyr::full_join, by = 'term') %>%
           stats::setNames(c('term', model_names))

    # add gof row identifier
    gof <- gof %>%
           dplyr::mutate(group = 'gof') %>%
           dplyr::select(group, term, names(.))

    # omit gof using regex
    if (!is.null(gof_omit)) {
        gof <- gof %>%
               dplyr::filter(!stringr::str_detect(term, gof_omit))
    }

    # gof_map: omit, reorder, rename
    gof <- gof[!gof$term %in% gof_map$raw[gof_map$omit],] # omit (black list)
	gof_names <- gof_map$clean[match(gof$term, gof_map$raw)] # rename
	gof_names[is.na(gof_names)] <- gof$term[is.na(gof_names)]
    gof$term <- gof_names
	idx <- match(gof$term, gof_map$clean) # reorder
    gof <- gof[order(idx, gof$term),] 

    # combine estimates and gof
    if (nrow(gof) > 0) {
        tab <- dplyr::bind_rows(est, gof)
    } else {
        tab <- est
    }

    # empty cells
    tab[is.na(tab)] <- ''

    # output
    return(tab)
}
